<?php



require_once 'Classes/Model/ObservationSet.php';
require_once 'Classes/Queries/ConeQuery.php';
require_once 'Classes/Queries/CalQuery.php';
require_once 'Classes/Pulsar/PulsarCatalog.php';



/**
 * @class ObservationSetFinder
 *
 * This algorithm uses the ConeQuery to find all the observations around the coordinates
 * in the parameters. Then we use the CalQuery to find all the calibrations done within
 * search_window minutes of each observation and return the results as a set of
 * ObservationSet objects
 **/

class ObservationSetFinder
{
  // PUBLIC

  public function __construct()
  {
    $this->parameters = null;
    $this->observation_sets = null;
    $this->database = null;
  }

  public function SetParameters( $s_parameters ) { $this->parameters = $s_parameters; }

  public function GetObservationSets() { return $this->observation_sets; }
  
  public function SetDatabase( $s_database ) { $this->database = $s_database; }

  public function Execute()
  {
    // If we were given a pulsar name, set the rajd and decjd to that pulsar

    $query = new ConeQuery();
    if( $this->parameters->GetPulsarName() != '' )
    {
      $catalog = new PulsarCatalog();
      $coords = $catalog->GetCoordsForPulsar( $this->parameters->GetPulsarName() );
      $this->parameters->SetCoordinates( $coords );
    }
    
    // perform the cone query to get a set of observations

    $cone_query = new ConeQuery();
    $cone_query->SetParameters( $this->parameters );
    $cone_query->SetDatabase( $this->database );
    $cone_query->Execute();
    $observations = $cone_query->GetObservations();
    
    // for each observation, get the associated calibrations

    $this->observation_sets = Array();
    foreach( $observations as $next_observation )
    {
      $new_obs_set = new ObservationSet();
      $new_obs_set->SetObservation( $next_observation );
      
      $cal_query = new CalQuery();
      $cal_query->SetStartTime( $this->DetermineCalStartTime( $next_observation ) );
      $cal_query->SetEndTime( $this->DetermineCalEndTime( $next_observation ) );
      $cal_query->SetDatabase( $this->database );
      $cal_query->Execute();
      $calibrations = $cal_query->GetCalibrations();
      
      $new_obs_set->SetCalibrations( $calibrations );
      
      $this->observation_sets[] = $new_obs_set;
    }
  }

  // PRIVATE

  private $parameters;
  private $observation_sets;
  private $database;
  
  private function DetermineCalStartTime( $observation )
  {
    $obs_time = $observation->GetMJD();

    // determine a min as a fraction of a day
    $half_min = 0.5 / (24.0 * 60.0);

    $min_mjd = new MJD();
    $min_mjd->FromString( $obs_time->ToString() - $this->parameters->GetCalsWindow() * $half_min );
    
    return $min_mjd;
  }
  
  private function DetermineCalEndTime( $observation )
  {
    $obs_time = $observation->GetMJD();

    // determine a min as a fraction of a day
    $half_min = 0.5 / (24.0 * 60.0);
	
    $max_mjd = new MJD();
    $max_mjd->FromString( $obs_time->ToString() + $this->parameters->GetCalsWindow() * $half_min );
    
    return $max_mjd;
  }
};





?>